/*------------------------------------------------------------------
   DEVCAPS2.C -- Abfrage der Charakteristika von Gerten (Version 2)
                 (c) Charles Petzold, 1998
------------------------------------------------------------------*/

#include <windows.h>
#include "resource.h"

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
void DoBasicInfo    (HDC, HDC, int, int) ;
void DoOtherInfo    (HDC, HDC, int, int) ;
void DoBitCodedCaps (HDC, HDC, int, int, int) ;

typedef struct
{
     int     iMask ;
     TCHAR * szDesc ;
}
BITS ;

#define IDM_DEVMODE      1000

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static TCHAR szAppName[] = TEXT ("DevCaps2") ;
     HWND         hwnd ;
     MSG          msg ;
     WNDCLASS     wndclass ;
     
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName ;
     wndclass.lpszClassName = szAppName ;
     
     if (!RegisterClass (&wndclass))
     {    // UNICODE-Compilierung ist die einzige realistische Fehlermglichkeit 
          MessageBox (NULL, TEXT ("Programm arbeitet mit Unicode und setzt Windows NT voraus!"), 
                      szAppName, MB_ICONERROR) ;
          return 0 ;
     }
     
     hwnd = CreateWindow (szAppName, NULL,
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;
     
     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;
     
     while (GetMessage (&msg, NULL, 0, 0))
     {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
     }
     return msg.wParam ;
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
     static TCHAR            szDevice[32], szWindowText[64] ;
     static int              cxChar, cyChar, nCurrentDevice = IDM_SCREEN,
                                             nCurrentInfo   = IDM_BASIC ;
     static DWORD            dwNeeded, dwReturned ;
     static PRINTER_INFO_4 * pinfo4 ;
     static PRINTER_INFO_5 * pinfo5 ;
     DWORD                   i ;
     HDC                     hdc, hdcInfo ;
     HMENU                   hMenu ;
     HANDLE                  hPrint ;
     PAINTSTRUCT             ps ;
     TEXTMETRIC              tm ;
     
     switch (message)
     {
     case WM_CREATE :
          hdc = GetDC (hwnd) ;
          SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT)) ;
          GetTextMetrics (hdc, &tm) ;
          cxChar = tm.tmAveCharWidth ;
          cyChar = tm.tmHeight + tm.tmExternalLeading ;
          ReleaseDC (hwnd, hdc) ;
                                           // weiter mit WM_SETTINGCHANGE
     case WM_SETTINGCHANGE:
          hMenu = GetSubMenu (GetMenu (hwnd), 0) ;
          
          while (GetMenuItemCount (hMenu) > 1)
               DeleteMenu (hMenu, 1, MF_BYPOSITION) ;

               // Liste aller lokalen Drucker und aller Netzwerk-Drucker
               // 
               // Beim ersten Aufruf von EnumPrinters geht es nur um die 
               // Grenermittlung (-> dwNeeded), beim zweiten um das Besetzen
               // der Felder einer (zuvor passend angelegten) Struktur
               // 
               // Die Druckernamen werden ins Men eingesetzt

          if (GetVersion () & 0x80000000)         // Windows 98
          {
               EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 5, NULL, 
                             0, &dwNeeded, &dwReturned) ;

               pinfo5 = malloc (dwNeeded) ;

               EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 5, (PBYTE) pinfo5,
                             dwNeeded, &dwNeeded, &dwReturned) ;

               for (i = 0 ; i < dwReturned ; i++)
               {
                    AppendMenu (hMenu, (i+1) % 16 ? 0 : MF_MENUBARBREAK, i + 1, 
                                pinfo5[i].pPrinterName) ;
               }
               free (pinfo5) ;
          }
          else                                    // Windows NT
          {
               EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 4, NULL, 
                             0, &dwNeeded, &dwReturned) ;

               pinfo4 = malloc (dwNeeded) ;
               EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 4, (PBYTE) pinfo4,
                             dwNeeded, &dwNeeded, &dwReturned) ;

               for (i = 0 ; i < dwReturned ; i++)
               {
                    AppendMenu (hMenu, (i+1) % 16 ? 0 : MF_MENUBARBREAK, i + 1, 
                                pinfo4[i].pPrinterName) ;
               }
               free (pinfo4) ;
          }
          
          AppendMenu (hMenu, MF_SEPARATOR, 0, NULL) ;
          AppendMenu (hMenu, 0, IDM_DEVMODE, TEXT ("Eigenschaften")) ;
          
          wParam = IDM_SCREEN ;
                                             // fall through
     case WM_COMMAND :
          hMenu = GetMenu (hwnd) ;
          
          if (LOWORD (wParam) == IDM_SCREEN ||         // IDM_SCREEN & Drucker
              LOWORD (wParam) < IDM_DEVMODE)       
          {
               CheckMenuItem (hMenu, nCurrentDevice, MF_UNCHECKED) ;
               nCurrentDevice = LOWORD (wParam) ;
               CheckMenuItem (hMenu, nCurrentDevice, MF_CHECKED) ;
          }
          else if (LOWORD (wParam) == IDM_DEVMODE)     // Eigenschaften
          {
               GetMenuString (hMenu, nCurrentDevice, szDevice,
                              sizeof (szDevice) / sizeof (TCHAR), MF_BYCOMMAND);
               
               if (OpenPrinter (szDevice, &hPrint, NULL))
               {
                    PrinterProperties (hwnd, hPrint) ;
                    ClosePrinter (hPrint) ;
               }
          }
          else                               // Wahlpunkte des Info-Mens
          {
               CheckMenuItem (hMenu, nCurrentInfo, MF_UNCHECKED) ;
               nCurrentInfo = LOWORD (wParam) ;
               CheckMenuItem (hMenu, nCurrentInfo, MF_CHECKED) ;
          }
          InvalidateRect (hwnd, NULL, TRUE) ;
          return 0 ;
          
     case WM_INITMENUPOPUP :
          if (lParam == 0)
               EnableMenuItem (GetMenu (hwnd), IDM_DEVMODE,
                    nCurrentDevice == IDM_SCREEN ? MF_GRAYED : MF_ENABLED) ;
          return 0 ;
          
     case WM_PAINT :
          lstrcpy (szWindowText, TEXT ("Gerte-Charakteristika: ")) ;
          
          if (nCurrentDevice == IDM_SCREEN)
          {
               lstrcpy (szDevice, TEXT ("DISPLAY")) ;
               hdcInfo = CreateIC (szDevice, NULL, NULL, NULL) ;
          }
          else
          {
               hMenu = GetMenu (hwnd) ;
               GetMenuString (hMenu, nCurrentDevice, szDevice,
                              sizeof (szDevice), MF_BYCOMMAND) ;
               hdcInfo = CreateIC (NULL, szDevice, NULL, NULL) ;
          }
          
          lstrcat (szWindowText, szDevice) ;
          SetWindowText (hwnd, szWindowText) ;
          
          hdc = BeginPaint (hwnd, &ps) ;
          SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT)) ;
          
          if (hdcInfo)
          {
               switch (nCurrentInfo)
               {
               case IDM_BASIC :
                    DoBasicInfo (hdc, hdcInfo, cxChar, cyChar) ;
                    break ;
                    
               case IDM_OTHER :
                    DoOtherInfo (hdc, hdcInfo, cxChar, cyChar) ;
                    break ;
                    
               case IDM_CURVE :
               case IDM_LINE :
               case IDM_POLY :
               case IDM_TEXT :
                    DoBitCodedCaps (hdc, hdcInfo, cxChar, cyChar,
                                    nCurrentInfo - IDM_CURVE);
                    break ;
               }
               DeleteDC (hdcInfo) ;
          }
          
          EndPaint (hwnd, &ps) ;
          return 0 ;
          
     case WM_DESTROY :
          PostQuitMessage (0) ;
          return 0 ;
     }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
}
     
void DoBasicInfo (HDC hdc, HDC hdcInfo, int cxChar, int cyChar)
{
     static struct
     {
          int     nIndex ;
          TCHAR * szDesc ;
     }
     info[] =
     {
          HORZSIZE,        TEXT ("HORZSIZE        Breite in Millimetern:"),
          VERTSIZE,        TEXT ("VERTSIZE        Hhe in Millimetern:"),
          HORZRES,         TEXT ("HORZRES         Breite in Pixeln:"),
          VERTRES,         TEXT ("VERTRES         Hhe in Rasterzeilen:"),
          BITSPIXEL,       TEXT ("BITSPIXEL       Farbbits pro Pixel:"),
          PLANES,          TEXT ("PLANES          Anzahl Farbebenen:"),
          NUMBRUSHES,      TEXT ("NUMBRUSHES      Anzahl Fllmuster:"),
          NUMPENS,         TEXT ("NUMPENS         Anzahl Zeichenstife:"),
          NUMMARKERS,      TEXT ("NUMMARKERS      Anzahl Marker:"),
          NUMFONTS,        TEXT ("NUMFONTS        Anzahl Schriftarten:"),
          NUMCOLORS,       TEXT ("NUMCOLORS       Anzahl Farben:"),
          PDEVICESIZE,     TEXT ("PDEVICESIZE     Umfang der Datenstruktur:"),
          ASPECTX,         TEXT ("ASPECTX         Relative Pixelbreite:"),
          ASPECTY,         TEXT ("ASPECTY         Relative Pixelhhe:"),
          ASPECTXY,        TEXT ("ASPECTXY        Relative Pixeldiagonale:"),
          LOGPIXELSX,      TEXT ("LOGPIXELSX      Punkte pro Inch, horizontal.:"),
          LOGPIXELSY,      TEXT ("LOGPIXELSY      Punkte pro Inch, vertikal:"),
          SIZEPALETTE,     TEXT ("SIZEPALETTE     Anzahl Paletteneintrge:"),
          NUMRESERVED,     TEXT ("NUMRESERVED     reservierte Paletteneintrge:"),
          COLORRES,        TEXT ("COLORRES        Tatschliche Farbenzahl:"),
          PHYSICALWIDTH,   TEXT ("PHYSICALWIDTH   Pixel horizontal pro Druckseite:"),
          PHYSICALHEIGHT,  TEXT ("PHYSICALHEIGHT  Pixel vertikal pro Druckseite:"),
          PHYSICALOFFSETX, TEXT ("PHYSICALOFFSETX X-Offset bei Druckseiten:"),
          PHYSICALOFFSETY, TEXT ("PHYSICALOFFSETY Y-Offset bei Druckseiten:") 
     } ;
     int   i ;
     TCHAR szBuffer[80] ;
          
     for (i = 0 ; i < sizeof (info) / sizeof (info[0]) ; i++)
          TextOut (hdc, cxChar, (i + 1) * cyChar, szBuffer,
               wsprintf (szBuffer, TEXT ("%-45s%8d"), info[i].szDesc,
                    GetDeviceCaps (hdcInfo, info[i].nIndex))) ;
}
    
void DoOtherInfo (HDC hdc, HDC hdcInfo, int cxChar, int cyChar)
{
     static BITS clip[] =
     {
          CP_RECTANGLE,    TEXT ("CP_RECTANGLE    Untersttzt Clipping:")
	 } ;
	 
	 static BITS raster[] =
	 {
          RC_BITBLT,       TEXT ("RC_BITBLT       Untersttzt einfaches BitBlt:"),
          RC_BANDING,      TEXT ("RC_BANDING      Bentigt separate Abschnitte:"),
          RC_SCALING,      TEXT ("RC_SCALING      Setzt Skalierung voraus:"),
          RC_BITMAP64,     TEXT ("RC_BITMAP64     Untersttzt Bitmaps >64K:"),
          RC_GDI20_OUTPUT, TEXT ("RC_GDI20_OUTPUT Arbeitet mit GDI-Version 2.0:"),
          RC_DI_BITMAP,    TEXT ("RC_DI_BITMAP    Kann DIB in Speicher kopieren:"),
          RC_PALETTE,      TEXT ("RC_PALETTE      Untersttzt Farbpaletten:"),
          RC_DIBTODEV,     TEXT ("RC_DIBTODEV     Kann Bitmaps konvertieren::"),
          RC_BIGFONT,      TEXT ("RC_BIGFONT      Untersttzt Schriftdateien >64K:"),
          RC_STRETCHBLT,   TEXT ("RC_STRETCHBLT   Kann StretchBlt ausfhren:"),
          RC_FLOODFILL,    TEXT ("RC_FLOODFILL    Kann Flchen fllen:"),
          RC_STRETCHDIB,   TEXT ("RC_STRETCHDIB   Kann DIBs skalieren:")
     } ;

     static TCHAR * szTech[] = { TEXT ("DT_PLOTTER (Vektor-Plotter)"),
                                 TEXT ("DT_RASDISPLAY (Raster-Bildschirm)"),
                                 TEXT ("DT_RASPRINTER (Raster-Drucker)"),
                                 TEXT ("DT_RASCAMERA (Raster-Kamera)"),
                                 TEXT ("DT_CHARSTREAM (Zeichenstrom)"),
                                 TEXT ("DT_METAFILE (Zwischendatei)"),
                                 TEXT ("DT_DISPFILE (Display-Datei)") } ;
     int            i ;
     TCHAR          szBuffer[80] ;
          
     TextOut (hdc, cxChar, cyChar, szBuffer,
          wsprintf (szBuffer, TEXT ("%-24s%04XH"), TEXT ("TREIBERVERSION:"),
               GetDeviceCaps (hdcInfo, DRIVERVERSION))) ;
          
     TextOut (hdc, cxChar, 2 * cyChar, szBuffer,
          wsprintf (szBuffer, TEXT ("%-24s%-40s"), TEXT ("TECHNOLOGIE:"), 
               szTech[GetDeviceCaps (hdcInfo, TECHNOLOGY)])) ;
          
     TextOut (hdc, cxChar, 4 * cyChar, szBuffer,
          wsprintf (szBuffer, TEXT ("CLIPCAPS (Clipping-Operationen)"))) ;
          
     for (i = 0 ; i < sizeof (clip) / sizeof (clip[0]) ; i++)
          TextOut (hdc, 9 * cxChar, (i + 6) * cyChar, szBuffer,
               wsprintf (szBuffer, TEXT ("%-45s %3s"), clip[i].szDesc,
                    GetDeviceCaps (hdcInfo, CLIPCAPS) & clip[i].iMask ?
                         TEXT ("Ja") : TEXT ("Nein"))) ;
          
     TextOut (hdc, cxChar, 8 * cyChar, szBuffer,
          wsprintf (szBuffer, TEXT ("RASTERCAPS (Raster-Fhigkeiten)"))) ;
      
     for (i = 0 ; i < sizeof (raster) / sizeof (raster[0]) ; i++)
          TextOut (hdc, 9 * cxChar, (i + 10) * cyChar, szBuffer,
               wsprintf (szBuffer, TEXT ("%-45s %3s"), raster[i].szDesc,
                    GetDeviceCaps (hdcInfo, RASTERCAPS) & raster[i].iMask ?
                         TEXT ("Ja") : TEXT ("Nein"))) ;
}

void DoBitCodedCaps (HDC hdc, HDC hdcInfo, int cxChar, int cyChar, int iType)
{
     static BITS curves[] =
     {
          CC_CIRCLES,    TEXT ("CC_CIRCLES    Kreise:"),
          CC_PIE,        TEXT ("CC_PIE        Kuchenstcke:"),
          CC_CHORD,      TEXT ("CC_CHORD      Kreisabschnitte/Sehnen:"),
          CC_ELLIPSES,   TEXT ("CC_ELLIPSES   Ellipsen:"),
          CC_WIDE,       TEXT ("CC_WIDE       breite Rnder:"),
          CC_STYLED,     TEXT ("CC_STYLED     Rand-Fllmuster:"),
          CC_WIDESTYLED, TEXT ("CC_WIDESTYLED   sowohl als auch:"),
          CC_INTERIORS,  TEXT ("CC_INTERIORS  Innenfllung von Flchen:")
     } ; 
     
     static BITS lines[] =
     {
          LC_POLYLINE,   TEXT ("LC_POLYLINE   Polygone:"),
          LC_MARKER,     TEXT ("LC_MARKER     Marker:"),
          LC_POLYMARKER, TEXT ("LC_POLYMARKER Polygon-Marker"),
          LC_WIDE,       TEXT ("LC_WIDE       breite Linien:"),
          LC_STYLED,     TEXT ("LC_STYLED     Linien-Fllmuster:"),
          LC_WIDESTYLED, TEXT ("LC_WIDESTYLED   sowohl als auch:"),
          LC_INTERIORS,  TEXT ("LC_INTERIORS  Innenfllung von Flchen:")
     } ;
     
     static BITS poly[] =
     {
          PC_POLYGON,     TEXT ("PC_POLYGON     Mit ALTERNATE gefllte Polygone:"),
          PC_RECTANGLE,   TEXT ("PC_RECTANGLE   Rechtecke:"),
          PC_WINDPOLYGON, TEXT ("PC_WINDPOLYGON Mit WINDING gefllte Polygone:"),
          PC_SCANLINE,    TEXT ("PC_SCANLINE    Zerlegung in Rasterzeilen:"),
          PC_WIDE,        TEXT ("PC_WIDE        breite Rnder:"),
          PC_STYLED,      TEXT ("PC_STYLED      Rand-Fllmuster:"),
          PC_WIDESTYLED,  TEXT ("PC_WIDESTYLED    sowohl als auch:"),
          PC_INTERIORS,   TEXT ("PC_INTERIORS   Innnenfllung von Flchen:")
     } ;
     
     static BITS text[] =
     {
          TC_OP_CHARACTER, TEXT ("TC_OP_CHARACTER Przision: einzelne Zeichen:"),
          TC_OP_STROKE,    TEXT ("TC_OP_STROKE      einzelne Striche:"),
          TC_CP_STROKE,    TEXT ("TC_CP_STROKE      abgeschnittene Striche (Clipping):"),
          TC_CR_90,        TEXT ("TC_CP_90        Zeichen um 90 Grad drehen:"),
          TC_CR_ANY,       TEXT ("TC_CR_ANY         mit beliebigen Winkeln:"),
          TC_SF_X_YINDEP,  TEXT ("TC_SF_X_YINDEP  Skalierung X und Y unabhngig:"),
          TC_SA_DOUBLE,    TEXT ("TC_SA_DOUBLE        doppelte Zeichengre:"),
          TC_SA_INTEGER,   TEXT ("TC_SA_INTEGER     nur mit ganzzahligen Werten:"),
          TC_SA_CONTIN,    TEXT ("TC_SA_CONTIN      mit Gleitkommazahlen:"),
          TC_EA_DOUBLE,    TEXT ("TC_EA_DOUBLE    Zeichenattribut fett:"),
          TC_IA_ABLE,      TEXT ("TC_IA_ABLE        kursiv:"),
          TC_UA_ABLE,      TEXT ("TC_UA_ABLE        unterstrichen:"),
          TC_SO_ABLE,      TEXT ("TC_SO_ABLE        durchgestrichen:"),
          TC_RA_ABLE,      TEXT ("TC_RA_ABLE      Bitmap-Schriften:"),
          TC_VA_ABLE,      TEXT ("TC_VA_ABLE      Vektor-Schriften:")
     } ;
     
     static struct
     {
          int     iIndex ;
          TCHAR * szTitle ;
          BITS    (*pbits)[] ;
          int     iSize ;
     }
     bitinfo[] =
     {
          CURVECAPS,  TEXT ("CURVCAPS (Kurven-Operationen)"),
               (BITS (*)[]) curves, sizeof (curves) / sizeof (curves[0]),
          LINECAPS,   TEXT ("LINECAPS (Linien-Operationen)"),
               (BITS (*)[]) lines, sizeof (lines) / sizeof (lines[0]),
          POLYGONALCAPS, TEXT ("POLYGONALCAPS (Polygon-Operationen)"),
               (BITS (*)[]) poly, sizeof (poly) / sizeof (poly[0]),
          TEXTCAPS,   TEXT ("TEXTCAPS (Text-Operationen)"),
               (BITS (*)[]) text, sizeof (text) / sizeof (text[0])
     } ;
     
     static TCHAR szBuffer[80] ;
     BITS         (*pbits)[] = bitinfo[iType].pbits ;
     int          i, iDevCaps = GetDeviceCaps (hdcInfo, bitinfo[iType].iIndex) ;
     
     TextOut (hdc, cxChar, cyChar, bitinfo[iType].szTitle,
              lstrlen (bitinfo[iType].szTitle)) ;
     
     for (i = 0 ; i < bitinfo[iType].iSize ; i++)
          TextOut (hdc, cxChar, (i + 3) * cyChar, szBuffer,
               wsprintf (szBuffer, TEXT ("%-55s %3s"), (*pbits)[i].szDesc,
                    iDevCaps & (*pbits)[i].iMask ? TEXT ("Ja") : TEXT ("Nein")));
}
