/*-----------------------------------------
   JUSTIFY2.C - Textformatierung, Version 2
                (c) Charles Petzold, 1998
  -----------------------------------------*/

#include <windows.h>
#include "resource.h"

#define OUTWIDTH 6       // Zeilenbreite der formatierten Ausgabe in Zoll
#define LASTCHAR 255     // ASCII-Code des letzten abzufragenden Zeichens

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

TCHAR szAppName[] = TEXT ("Justify2") ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     HWND     hwnd ;
     MSG      msg ;
     WNDCLASS wndclass ;
     
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName ;
     wndclass.lpszClassName = szAppName ;

     if (!RegisterClass (&wndclass))
     {    // UNICODE-Compilierung ist die einzige realistische Fehlermglichkeit 
          MessageBox (NULL, TEXT ("Programm arbeitet mit Unicode und setzt Windows NT voraus!"), 
                      szAppName, MB_ICONERROR) ;
          return 0 ;
     }
     
     hwnd = CreateWindow (szAppName, TEXT ("Textformatierung, Version 2"),
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;
     
     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;
     
     while (GetMessage (&msg, NULL, 0, 0))
     {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
     }
     return msg.wParam ;
}

void DrawRuler (HDC hdc, RECT * prc)
{
     static int iRuleSize [16] = { 360, 72, 144, 72, 216, 72, 144, 72,
                                   288, 72, 144, 72, 216, 72, 144, 72 } ;
     int        i, j ;
     POINT      ptClient ;
     
     SaveDC (hdc) ;
     
     // Koordinatensytem "logische Twips"
     SetMapMode (hdc, MM_ANISOTROPIC) ;
     SetWindowExtEx (hdc, 1440, 1440, NULL) ;
     SetViewportExtEx (hdc, GetDeviceCaps (hdc, LOGPIXELSX),
                            GetDeviceCaps (hdc, LOGPIXELSY), NULL) ;
     
     // Ursprung = 0.5 Zoll von der oberen linken Ecke entfernt
     SetWindowOrgEx (hdc, -720, -720, NULL) ;
     
     // Rechter Rand = 0.25 Zoll von der rechten Seite     
     ptClient.x = prc->right ;
     ptClient.y = prc->bottom ;
     DPtoLP (hdc, &ptClient, 1) ;
     ptClient.x -= 360 ;
     
     // Lineal zeichnen
     MoveToEx (hdc, 0,               -360, NULL) ;
     LineTo   (hdc, OUTWIDTH * 1440, -360) ;
     MoveToEx (hdc, -360,               0, NULL) ;
     LineTo   (hdc, -360,      ptClient.y) ;
     
     for (i = 0, j = 0 ; i <= ptClient.x && i <= OUTWIDTH * 1440 ; i += 1440 / 16, j++)
     {
          MoveToEx (hdc, i, -360, NULL) ;
          LineTo   (hdc, i, -360 - iRuleSize [j % 16]) ;
     }
     
     for (i = 0, j = 0 ; i <= ptClient.y ; i += 1440 / 16, j++)
     {
          MoveToEx (hdc, -360, i, NULL) ;
          LineTo   (hdc, -360 - iRuleSize [j % 16], i) ;
     }
     
     RestoreDC (hdc, -1) ;
}

/*----------------------------------------------------------------------
   GetCharDesignWidths:  Liefert die Zeichenbreiten der Schrift unskaliert
  ----------------------------------------------------------------------*/

UINT GetCharDesignWidths (HDC hdc, UINT uFirst, UINT uLast, int * piWidths)
{
     HFONT             hFont, hFontDesign ;
     LOGFONT           lf ;
     OUTLINETEXTMETRIC otm ;

     hFont = GetCurrentObject (hdc, OBJ_FONT) ;
     GetObject (hFont, sizeof (LOGFONT), &lf) ;

     // Outline-Textmetrik abfragen. Hier geht es ausschlielich um die Gre 
     // des vom Schriftdesigners verwendeten Koordinatensystems, der Gerte-
     // kontext spielt dabei keine Rolle
     otm.otmSize = sizeof (OUTLINETEXTMETRIC) ;
     GetOutlineTextMetrics (hdc, sizeof (OUTLINETEXTMETRIC), &otm) ;

     // Neue Schrift mit einer logischen Gre anlegen, die dem 
     // TrueType-intenen Koordinatensystem exakt entspricht
     lf.lfHeight = - (int) otm.otmEMSquare ;  // Zeichenhhe, nicht Zeilenabstand!
     lf.lfWidth  = 0 ;
     hFontDesign = CreateFontIndirect (&lf) ;

     // Schrift in den Gertekontext einsetzen und Zeichenbreiten ermitteln
     SaveDC (hdc) ;
     SetMapMode (hdc, MM_TEXT) ;
     SelectObject (hdc, hFontDesign) ;

     GetCharWidth (hdc, uFirst, uLast, piWidths) ;
     SelectObject (hdc, hFont) ;
     RestoreDC (hdc, -1) ;

     // Aufrumen
     DeleteObject (hFontDesign) ;

     return otm.otmEMSquare ;
}

/*---------------------------------------------------------------------------
   GetScaledWidths:  Liefert die Zeichenbreiten fr die gewhlte Schriftgre
      als Gleitkommawerte
  ---------------------------------------------------------------------------*/

void GetScaledWidths (HDC hdc, double * pdWidths)
{
     double  dScale ;
     HFONT   hFont ;
     int     aiDesignWidths [LASTCHAR + 1] ;
     int     i ;
     LOGFONT lf ;
     UINT    uEMSquare ;

     // Outline-Zeichenbreiten abfragen
     uEMSquare = GetCharDesignWidths (hdc, 0, LASTCHAR, aiDesignWidths) ;

    // Abfrage der aktuell in den Kontext eingesetzten Schrift
     hFont = GetCurrentObject (hdc, OBJ_FONT) ;
     GetObject (hFont, sizeof (LOGFONT), &lf) ;

     // Zeichenbreiten entsprechend skalieren und als Gleitkomma-Array speichern
     dScale = (double) -lf.lfHeight / (double) uEMSquare ;

     for (i = 0 ; i <= LASTCHAR ; i++)
          pdWidths[i] = dScale * aiDesignWidths[i] ;
}

/*----------------------------------------------------------------
   GetTextExtentFloat:  Berechnet Textbreiten mit Gleitkommawerten
  ----------------------------------------------------------------*/

double GetTextExtentFloat (double * pdWidths, PTSTR psText, int iCount)
{
     double dWidth = 0 ;
     int    i ;

     for (i = 0 ; i < iCount ; i++)
          dWidth += pdWidths [psText[i]] ;

     return dWidth ;
}

/*--------------------------------------------------------------------------------------
   Justify mit Breitenberechnung ber die Originaldaten der Schrift, ohne Rundungsfehler
  --------------------------------------------------------------------------------------*/

void Justify (HDC hdc, PTSTR pText, RECT * prc, int iAlign)
{
     double dWidth, adWidths[LASTCHAR + 1] ;
     int    xStart, yStart, cSpaceChars ;
     PTSTR  pBegin, pEnd ;
     SIZE   size ;

     // Array adWidths mit skalierten Gleitkomma-Zeichenbreiten besetzen
     GetScaledWidths (hdc, adWidths) ;

     yStart = prc->top ;
     do                            // fr jede Textzeile
     {
          cSpaceChars = 0 ;        // bis jetzt 0 Leerzeichen

          while (*pText == ' ')    // fhrende Leerzeichen berspringen
               pText++ ; 
          pBegin = pText ;         // Zeiger auf den Zeilenanfang
          
          do                       // solange, bis die Zeile bearbeitet ist
          {
               pEnd = pText ;      // aktuelles Zeilenende (beim 1. Mal = Zeilenstart)

               // Suche nach dem ersten/nchsten Leerzeichen
               while (*pText != '\0' && *pText++ != ' ') ;

               if (*pText == '\0')
                    break ;

               // Leerzeichen gefunden. Wie viel Platz braucht die Zeile jetzt?
               cSpaceChars++ ;
               dWidth = GetTextExtentFloat (adWidths, pBegin, pText - pBegin - 1) ;
          }
          while (dWidth < (double) (prc->right - prc->left)) ;
          
          cSpaceChars-- ;               // dieses Leerzeichen nicht mitzhlen
          
          while (*(pEnd - 1) == ' ')    // und weitere Leerzeichen am Zeilenende raus
          {
               pEnd-- ;
               cSpaceChars-- ;
          }

         // Textende erreicht oder keine Leerzeichen gefunden?
          if (*pText == '\0' || cSpaceChars <= 0)
               pEnd = pText ;

         // Platzbedarf erst jetzt ber die Standardfunktion abfragen
          GetTextExtentPoint32(hdc, pBegin, pEnd - pBegin, &size) ;
          
          switch (iAlign)               // Ausrichtung == xStart
          {
          case IDM_ALIGN_LEFT:
               xStart = prc->left ;
               break ;

          case IDM_ALIGN_RIGHT:
               xStart = prc->right - size.cx ;
               break ;

          case IDM_ALIGN_CENTER:
               xStart = (prc->right + prc->left - size.cx) / 2 ;
               break ;
               
          case IDM_ALIGN_JUSTIFIED:
               if (*pText != '\0' && cSpaceChars > 0)
                    SetTextJustification (hdc, prc->right - prc->left - size.cx, cSpaceChars) ;
               xStart = prc->left ;
               break ;
          }
          // Ausgabe einer Zeile
          TextOut (hdc, xStart, yStart, pBegin, pEnd - pBegin) ;

          // Vorbereitungen fr die nchste Zeile
          SetTextJustification (hdc, 0, 0) ;
          yStart += size.cy ;
          pText = pEnd ;
     }
     while (*pText && yStart < prc->bottom - size.cy) ;
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
     static CHOOSEFONT cf ;
     static DOCINFO    di = { sizeof (DOCINFO), TEXT ("Justify2: Printing") } ;
     static int        iAlign = IDM_ALIGN_LEFT ;
     static LOGFONT    lf ;
     static PRINTDLG   pd ;
     static TCHAR      szText[] = { 
        TEXT ("Call me Ishmael. Some years ago -- never mind how long precisely -- having little ")
        TEXT ("or no money in my purse, and nothing particular to interest me on shore, I ")
        TEXT ("thought I would sail about a little and see the watery part of the world. It is ")
        TEXT ("a way I have of driving off the spleen, and regulating the circulation. Whenever ")
        TEXT ("I find myself growing grim about the mouth; whenever it is a damp, drizzly ")
        TEXT ("November in my soul; whenever I find myself involuntarily pausing before ")
        TEXT ("coffin warehouses, and bringing up the rear of every funeral I meet; and ")
        TEXT ("especially whenever my hypos get such an upper hand of me, that it requires a ")
        TEXT ("strong moral principle to prevent me from deliberately stepping into the ")
        TEXT ("street, and methodically knocking people's hats off -- then, I account it ")
        TEXT ("high time to get to sea as soon as I can. This is my substitute for pistol ")
        TEXT ("and ball. With a philosophical flourish Cato throws himself upon his sword; I ")
        TEXT ("quietly take to the ship. There is nothing surprising in this. If they but ")
        TEXT ("knew it, almost all men in their degree, some time or other, cherish very nearly ")
        TEXT ("the same feelings towards the ocean with me.") } ;
     BOOL              fSuccess ;
     HDC               hdc, hdcPrn ;
     HMENU             hMenu ;
     int               iSavePointSize ;
     PAINTSTRUCT       ps ;
     RECT              rect ;
     
     switch (message)
     {
     case WM_CREATE:
          // CHOOSEFONT-Struktur mit Times New Roman initialisieren
          hdc = GetDC (hwnd) ;
          lf.lfHeight = - GetDeviceCaps (hdc, LOGPIXELSY) / 6 ;
          lf.lfOutPrecision = OUT_TT_ONLY_PRECIS ;
          lstrcpy (lf.lfFaceName, TEXT ("Times New Roman")) ;
          ReleaseDC (hwnd, hdc) ;

          cf.lStructSize    = sizeof (CHOOSEFONT) ;
          cf.hwndOwner      = hwnd ;
          cf.hDC            = NULL ;
          cf.lpLogFont      = &lf ;
          cf.iPointSize     = 120 ;
          // Auswahlmglichkeiten auf TrueType-Schriften beschrnkt
          cf.Flags          = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS | CF_TTONLY | CF_EFFECTS ;
          cf.rgbColors      = 0 ;
          cf.lCustData      = 0 ;
          cf.lpfnHook       = NULL ;
          cf.lpTemplateName = NULL ;
          cf.hInstance      = NULL ;
          cf.lpszStyle      = NULL ;
          cf.nFontType      = 0 ;      
          cf.nSizeMin       = 0 ;
          cf.nSizeMax       = 0 ;
  
          return 0 ;

     case WM_COMMAND:
          hMenu = GetMenu (hwnd) ;
          
          switch (LOWORD (wParam))
          {
          case IDM_FILE_PRINT:
               // Drucker-Gertekontext
               pd.lStructSize = sizeof (PRINTDLG) ;
               pd.hwndOwner   = hwnd ;
               pd.Flags       = PD_RETURNDC | PD_NOPAGENUMS | PD_NOSELECTION ;

               if (!PrintDlg (&pd))
                    return 0 ;

               if (NULL == (hdcPrn = pd.hDC))
               {
                    MessageBox (hwnd, TEXT ("Kein Druckerkontext verfgbar"),
                                szAppName, MB_ICONEXCLAMATION | MB_OK) ;
                    return 0 ;
               }

               // Rnder fr eine Breite von OUTWIDTH Zoll setzen
               rect.left  = (GetDeviceCaps (hdcPrn, PHYSICALWIDTH) -
                             GetDeviceCaps (hdcPrn, LOGPIXELSX) * OUTWIDTH) / 2 
                           - GetDeviceCaps (hdcPrn, PHYSICALOFFSETX) ;
               
               rect.right = rect.left +  GetDeviceCaps (hdcPrn, LOGPIXELSX) * OUTWIDTH ;

               // Oberer und unterer Rand auf 1 Zoll
               rect.top    = GetDeviceCaps (hdcPrn, LOGPIXELSY) -
                             GetDeviceCaps (hdcPrn, PHYSICALOFFSETY) ; 
               rect.bottom = GetDeviceCaps (hdcPrn, PHYSICALHEIGHT) - 
                             GetDeviceCaps (hdcPrn, LOGPIXELSY) -
                             GetDeviceCaps (hdcPrn, PHYSICALOFFSETY) ;

               // Formatierung und Ausgabe auf den Drucker
               SetCursor (LoadCursor (NULL, IDC_WAIT)) ;
               ShowCursor (TRUE) ;

               fSuccess = FALSE ;

               if ((StartDoc (hdcPrn, &di) > 0) && (StartPage (hdcPrn) > 0))
               {
                    // Schriftgre (lfHeight) auf den Drucker umsetzen
                    iSavePointSize = lf.lfHeight ;
                    lf.lfHeight = -(GetDeviceCaps (hdcPrn, LOGPIXELSY) * cf.iPointSize) / 720 ;

                    SelectObject (hdcPrn, CreateFontIndirect (&lf)) ;
                    lf.lfHeight = iSavePointSize ;

                    // Textfarbe setzen
                    SetTextColor (hdcPrn, cf.rgbColors) ;
               
                    // Zeilenumbruch und Ausgabe
                    Justify (hdcPrn, szText, &rect, iAlign) ;

                    if (EndPage (hdcPrn) > 0)
                    {
                         fSuccess = TRUE ;
                         EndDoc (hdcPrn) ;
                    }
               }
               ShowCursor (FALSE) ;
               SetCursor (LoadCursor (NULL, IDC_ARROW)) ;

               DeleteDC (hdcPrn) ;

               if (!fSuccess)
                    MessageBox (hwnd, TEXT ("Fehler beim Drucken"),
                                szAppName, MB_ICONEXCLAMATION | MB_OK) ;
               return 0 ;

     case IDM_FONT:
               if (ChooseFont (&cf))
                    InvalidateRect (hwnd, NULL, TRUE) ;
               return 0 ;
               
          case IDM_ALIGN_LEFT:
          case IDM_ALIGN_RIGHT:
          case IDM_ALIGN_CENTER:
          case IDM_ALIGN_JUSTIFIED:
               CheckMenuItem (hMenu, iAlign, MF_UNCHECKED) ;
               iAlign = LOWORD (wParam) ;
               CheckMenuItem (hMenu, iAlign, MF_CHECKED) ;
               InvalidateRect (hwnd, NULL, TRUE) ;
               return 0 ;
          }
          return 0 ;

     case WM_PAINT:
          hdc = BeginPaint (hwnd, &ps) ;
          
          GetClientRect (hwnd, &rect) ;
          DrawRuler (hdc, &rect) ;
          
          rect.left  += GetDeviceCaps (hdc, LOGPIXELSX) / 2 ;
          rect.top   += GetDeviceCaps (hdc, LOGPIXELSY) / 2 ;
          rect.right = rect.left + OUTWIDTH * GetDeviceCaps (hdc, LOGPIXELSX) ;

          SelectObject (hdc, CreateFontIndirect (&lf)) ;
          SetTextColor (hdc, cf.rgbColors) ;
          
          Justify (hdc, szText, &rect, iAlign) ;
          
          DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT))) ;
          EndPaint (hwnd, &ps) ;
          return 0 ;
          
     case WM_DESTROY:
          PostQuitMessage (0) ;
          return 0 ;
     }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
}
